package jartege ; 

import java.lang.reflect.* ; 

class ErrorMessage {

   /**
    * Negative weight error.
    */

   static void negativeWeight(String methodName, double weight) {
      System.err.println(
         methodName + ": weight should be positive or null" + 
         " (weight = " + weight + ").") ; 
      System.exit(1) ; 
   }

   /**
    * Class not found error.
    */

   static void classNotFound(String className) {
      System.err.println(
         "Class " + className + " not found.\n" + 
         "Check:\n" + 
         "   - class name\n" + 
         "   - class path\n" + 
         "   - existence of file " + Chaine.nom(className) + ".class\n") ; 
      System.exit(1) ; 
   }

   /**
    * Class linkage error.
    */

   static void classLinkageError(String className) {
      System.err.println("Class " + className + " impossible to link.") ;
      System.err.println("Check the class name and the classpath.") ;
      int t = className.length() ;
      if (t <= 1 || className.substring(t - 2, t - 1).equals(".")) {
         System.err.println(
            "Bug in j2se: class name must have more than one character.") ;
      }
      System.exit(1) ;
   }

   /**
    * Package not found error.
    */

   static void packageNotFound(String packageName) {
      System.err.println("Package " + packageName + " not found.") ;
      System.err.println("Check the package name and the class path.") ;
      System.exit(1) ;
   }

   /**
    * Class should not be abstract.
    */

   static void shouldNotBeAbstract(String className) {
      System.err.println("Class " + className +
         "should not be abstract.") ;
      System.exit(1) ;
   }

   /**
    * Method not found error.
    */

   static void methodNotFound
         (String where, String methodName, String className) {
      System.err.println(
         where + ": Method " + methodName +
         " not found in class " + className + ".") ;
      System.exit(1) ;
   }

   /**
    * Method not found.
    */

   static void methodNotFound
         (String where, 
          String methodName, 
          String[] theSignature, 
          String className) {
      System.err.print(
         where + ": Method " + methodName +
         "(") ;
      for (int i = 0 ; i < theSignature.length ; i++) {
         System.err.print(theSignature[i]) ;
         if (i != theSignature.length - 1) {
            System.err.print(", ") ;
         }
      }
      System.err.println(") not found in class " + className) ;
      System.exit(1) ;
   }

   /**
    * Constructor not found.
    */

   static void constructorNotFound
         (String where, String constructorName, String[] theSignature) {
      System.err.print(
         where + ": Constructor " + constructorName +
         "(") ;
      for (int i = 0 ; i < theSignature.length ; i++) {
         System.err.print(theSignature[i]) ;
         if (i != theSignature.length - 1) {
            System.err.print(", ") ;
         }
      }
      System.err.println(") not found.") ;
      System.exit(1) ;
   }

   /**
    * Field not found.
    */

   static void fieldNotFound
         (String where, String fieldName, String className) {
      System.err.println(
         where + ": field " + fieldName +
         " not found in class " + className) ;
      System.exit(1) ;
   }

   /**
    * Operation get not found for field.
    */

   static void getNotFound
         (String where, String fieldName, String className) {
      System.err.println(
         where + ": no operation get found for field " + fieldName + 
         " in class " + className + ".") ;
      System.err.println("Check name and accessibility of the field.") ; 
      System.exit(1) ;
   }

   /**
    * Operation set not found for field.
    */

   static void setNotFound
         (String where, String fieldName, String className) {
      System.err.println(
         where + ": no operation set found for field " + fieldName +
         " in class " + className) ;
      System.err.println("Check name and accessibility of the field.") ; 
      System.err.println("A final field cannot be set.") ; 
      System.exit(1) ;
   }


   /**
    * Input / output error in writing a file.
    */

   static void ioWriteError(String fileName) {
      System.err.println(
         "I/O error in writing file " + fileName + ".") ; 
      System.exit(1) ;
   }

   /**
    * No constructor () in class.
    */

   static void noConstructorInClass(String className) {
      System.err.println(
         "Class " + className +
         " should contain the constructor " + className + "().") ;
      System.exit(1) ;
   }


   /**
    * Invocation error.
    */

   static void invocation(Member member, Throwable throwable) {
      if (throwable instanceof InvocationTargetException) {
         InvocationTargetException invocationException = 
            (InvocationTargetException) throwable ; 
         Throwable target = invocationException.getTargetException() ; 
         System.err.println(
            "Execution of " + member.getName() + 
            " in class " + member.getDeclaringClass() + 
            " has completed abruptelty throwing exception " + 
            target + ".\n") ; 
         target.printStackTrace() ; 
      } else {
         System.err.println(
            "Invocation of " + member.getName() +
            " in class " + member.getDeclaringClass() + 
            " has trownn " + throwable + ".\n") ; 
         throwable.printStackTrace() ;
      }
      System.exit(1) ; 
   }

   /**
    * Class should extend TestCase.
    */

   static void shouldExtendTestCase(String className) {
      System.err.println(
         "Class " + className + 
         " should extend " + Constantes.TestCase + ".") ;
      System.exit(1) ;
   }

   /**
    * No setUp() method error.
    */

   static void noSetUp(String className) {
      System.err.println(
         "Class " + className + " should contain a setUp() method.") ; 
      System.exit(1) ; 
   }

   
   /**
    * No tearDown() method error.
    */
   static void noTearDown(String className) {
      System.err.println(
         "Class " + className + " should contain a tearDown() method.") ;
      System.exit(1) ;
   }

   /**
    * Class should be public.
    */
   static void classShouldBePublic(String className) {
      System.err.println(
         "Class " + className + " should be public.") ; 
   }

   /**
    * Incorrect package name.
    */

   static void incorrectPackageName(String packageName) {
      System.out.println(
         "setPackage: incorrect package name (" + 
         packageName + ")") ; 
      System.exit(1) ; 
   }

   /**
    * No class under test defined.
    */

   static void noClassUnderTest() {
      System.out.println(
         "generate: there is no class under test.") ; 
      System.exit(1) ;
   }

   

}

