package jartege ; 

import java.util.* ; 
import java.io.* ; 
import jartege.* ; 
import java.lang.reflect.* ; 

/**
 * Class that allows one to generate random tests. 
 */

class Generateur {

   // The class tester associated with this generator
   ClassTester classTester ; 

   // Le nom de la classe gnre
   String className ; 

   // The generated file
   FileWriter file ; 

   // The file name
   String fileName ; 

   // The setUp method of the class to extend, or null
   Method setUp ; 

   // The tearDown method of the class to extend, or null
   Method tearDown ; 

   Class toExtendClass ; 


   /**
    * Constructs a new test class generator.
    *
    * @param classTester the associated classTester for this generator. 
    * @param className the name of the generated test class.
    */
    
   public Generateur
         (ClassTester classTester, String className) {
      this.classTester = classTester ; 
      this.className = className ; 
      fileName = className + ".java" ; 
      String dir = classTester.getDir() ; 
      if (!dir.equals("")) {
         fileName = dir + "/" + fileName ; 
      }
      File fich = new File(fileName) ; 
      try {
         file = new FileWriter(fich) ; 
      } catch (IOException io) {
         ErrorMessage.ioWriteError(fileName) ; 
      }

      toExtendClass = classTester.getToExtendClass() ; 

      Class testCase = Util.forName(Constantes.TestCase) ; 

      if (toExtendClass == null && classTester.getForJUnit()) {
         toExtendClass = testCase ; 
         classTester.addImportedClass(Constantes.TestCase) ; 
      }


      // Calcul de la methode setUp et tearDown

      setUp = null ; 
      tearDown = null ; 

      if (toExtendClass != null) {

         Set methods = Constituant.getAllVisibleMethods
                            (toExtendClass, toExtendClass) ;  

         Iterator it = methods.iterator() ; 

         while (it.hasNext()) {
            Method method = (Method) it.next() ; 
            if (method.getName().equals("setUp") &&
                method.getParameterTypes().length == 0) {
               setUp = method ; 
               setUp.setAccessible(true) ; 
            }
            if (method.getName().equals("tearDown") &&
                method.getParameterTypes().length == 0) {
               tearDown = method ;
               tearDown.setAccessible(true) ; 
            }
         }

         if (classTester.getClasses().contains(toExtendClass)) {
            if (setUp != null) {
               classTester.changeMethodWeight
                  (toExtendClass.getName(), "setUp", new String[0], 0) ; 
            }
            if (tearDown != null) {
               classTester.changeMethodWeight
                  (toExtendClass.getName(), "tearDown", new String[0], 0) ;
            }
         }
      }

      classTester.addImportedClass(Constantes.JMLEntryPreconditionError) ; 

      if (classTester.getForJUnit()) {
         if (!testCase.isAssignableFrom(toExtendClass)) { 
            ErrorMessage.shouldExtendTestCase(toExtendClass.getName()) ; 
         }
         classTester.addImportedClass(Constantes.JMLAssertionError) ; 
         // On vrifie qu'il existe une mthode setUp et une mthode tearDown
         if (setUp == null) {
            ErrorMessage.noSetUp(classTester.getToExtendClass().getName()) ; 
         }
         if (tearDown == null) {
            ErrorMessage.noTearDown(classTester.getToExtendClass().getName()) ; 
         }

      }

   }


   // Gnration de l'entte du fichier
   private void writeHeader() throws IOException {

      String packageDef = "" ; 
      String packageName = classTester.getPackage() ; 

      if (packageName != null) {
         packageDef = "package " + packageName + " ; \n\n" ; 
      }

      Chaine.ecrire(file, 
         "/** \n" +
         " *  Test class " + className + " generated by jartege. \n" + 
         " */ \n\n" + 
         packageDef) ; 

      Chaine.ecrire(file,
         "import java.io.FileWriter ; \n" +
         "import java.io.IOException ; \n") ; 
   

      Iterator it ; 
      it = classTester.getImportedPackages().iterator() ; 
      while (it.hasNext()) {
         String ImportedPackageName = (String) it.next() ; 
         Chaine.ecrire(file, 
            "import " + ImportedPackageName + ".* ; \n") ; 
      }

      it = classTester.getImportedClasses().iterator() ;
      while (it.hasNext()) {
         Class clas = (Class) it.next() ;
         Chaine.ecrire(file,
            "import " + clas.getName() + " ; \n") ;
      }

   }
      
   // Gnration de numberOfMethodCalls appels de mthode
   private void generateNCalls(int numberOfMethodCalls) 
   throws IOException {
      List lesClasses = classTester.lesClasses() ; 
      for (int i = 1 ; i <= numberOfMethodCalls ; i++) {
         Classe classe = (Classe) ElementAPoids.choisir(lesClasses) ; 
         if (classe == null) {
            ErrorMessage.noClassUnderTest() ; 
         }
         Trace.afficher(2, "(" + i + ") " + classe.nom) ; 
         classe.generer(file) ;  
      }
   }

   // Gnration de l'entte de la classe
   void writeClass() throws IOException {
      Chaine.ecrire(file, 
         "\n" +
         "public class " + className) ; 
      if (toExtendClass != null) {
         Chaine.ecrire(file, " extends " + toExtendClass.getName()) ; 
      }
      Chaine.ecrire(file, " { \n\n") ; 
   }

   // Gnration du programme principal
   // et de la mthode qui affiche un message d'erreur.
   void writeMain(int numberOfTests, int numberOfMethodCalls) throws IOException {

      if (!classTester.getForJUnit()) {

         Chaine.ecrire(file, 
            "   // Number of errors detected \n" + 
            "   private static int _fail ; \n\n" + 
            "   // Number of inconclusive tests \n" + 
            "   private static int _inconclusive ; \n\n" + 
            "   // Result file \n" + 
            "   private static FileWriter _file ; \n\n") ; 

         Chaine.ecrire(file,
            "   /** \n" +
            "    * Main program. \n" +
            "    */ \n" + 
            "\n" +
            "   public static void main(String[] args) throws Exception {" +
            "\n" + 
            "      _fail = 0 ; \n" + 
            "      _inconclusive = 0 ; \n" + 
            "      _file = new FileWriter(\"" + className + ".dat\") ; \n" +
            "      _file.write(" + 
            "\"// Test file: " + className + " generated by Jartege" +
            "\\n\\n\" + \n " + 
            "         \"" + className + " = [\") ; \n\n" + 
            "      " + className + 
            " _tester = new " + className + 
            "() ; \n") ; 

         for (int i = 1 ; i <= numberOfTests ; i++) {
            Chaine.ecrire(file, "      _tester.test" + i + "() ; \n") ; 
         }

         Chaine.ecrire(file, "\n" +
            "      _file.write(\"] \\n\\n\" + \n" +
            "         \"// Number of tests: " + numberOfTests + "\\n\" + \n" +
            "         \"// Number of method calls: " + numberOfMethodCalls + "\\n\" + \n" +
            "         \"// Number of errors: \" + _fail + \"\\n\" + \n" +
            "         \"// Number of inconclusive tests: \"" +
            " + _inconclusive + \"\\n\" + \n" + 
            "         \"// Number of pass tests: \"" + 
            " + (" + numberOfTests + 
            " - _fail - _inconclusive) + \"\\n\" ) ; \n\n") ; 

         Chaine.ecrire(file,  
            "      _file.write(\"intervalle = [0:1:" + 
            numberOfMethodCalls + 
            "] \\n\" + \n" +
            "         \"xbasc() \\n\" +\n" +
            "         \"histplot(intervalle, " + 
            className + 
            ", 3) \\n\");  \n\n") ; 

         Chaine.ecrire(file, 
            "      _file.close() ; \n\n") ;  
       

         Chaine.ecrire(file, "\n" +
            "      System.out.println(\"\\n\" + \n" + 
            "         \"Test file: " + className + "\\n\" + \n" + 
            "         \"Number of tests: " + numberOfTests + "\\n\" + \n" + 
            "         \"Number of errors: \" + _fail + \"\\n\" + \n" + 
            "         \"Number of inconclusive tests: \"" + 
            " + _inconclusive + \"\\n\") ; \n\n" + 
            "      System.exit(0) ; \n" + 
            "   } // main\n\n" ) ;

         Chaine.ecrire(file, 
            "   /** \n" + 
            "    * An error has been detected. \n" +
            "    */ \n" +
            "\n" +  
            "   private void error(Throwable except, int i, int _nb_calls) \n" +
            "   throws IOException {\n" +
            "      if (except instanceof JMLEntryPreconditionError) { \n" +
            "         _inconclusive++ ; \n" +
            "         System.out.println(\"Inconclusive test for" + 
                         " method test\" + i) ; \n" +
            "       } else { \n" + 
            "         _fail++ ; \n" + 
            "         System.out.print(\"\\n\" + \n" +
            "            _fail + \") \" + \"Error detected in class " +
                         className + "\" + \n" +
            "            \" by method test\" + i + \": \\n        \") ;\n" +
//            "         System.out.println(\"_nb_calls = \" + _nb_calls) ; \n" +
            "         if (_fail != 1) { \n" + 
            "            _file.write(\", \") ; \n" + 
            "         } \n" + 
            "         _file.write(_nb_calls + \"\") ; \n" +
            "         except.printStackTrace() ; \n" + 
            "       } \n" + 
            "   } \n\n") ; 
      }
   }

   // Excution de la mthode setUp
   void executeSetUp(Object unObjet) {

      Class toExtendClass = classTester.getToExtendClass() ; 

      if (toExtendClass != null) {

         if (setUp != null) {
            // Invocation de la mthode setUp()
            try {
               setUp.invoke(unObjet, new Object[0]) ; 
            } catch (Throwable throwable) {
               ErrorMessage.invocation(setUp, throwable) ; 
            }
         }

         // Mise  jour des variables de la classe tendue dans 
         // l'ensemble des objets de la classe. 

         Set lesChamps = Constituant.getAllVisibleFields
                            (toExtendClass, toExtendClass) ; 

         Iterator it = lesChamps.iterator() ; 

         while (it.hasNext()) {
            Field champ = (Field) it.next() ; 
            champ.setAccessible(true) ; 
            Classe classe = classTester.leDictionnaire.getClasse(champ.getType()) ; 
            Object valeur = null ; 
            try {
               valeur = champ.get(unObjet) ; 
            } catch (Throwable throwable) {
               ErrorMessage.invocation(champ, throwable) ;
            }

            Valeur val = new Valeur(champ.getName(), classe, valeur) ; 
            classe.ajouter(val) ; 
         }
         Classe classe = classTester.leDictionnaire.getClasse(toExtendClass) ; 
         classe.ajouter(new Valeur("this", classe, unObjet)) ; 
      }
   }

   // Excution de la mthode tearDown
   void executeTearDown(Object unObjet) {
      if (classTester.getToExtendClass() != null) {
         if (tearDown != null) {   
            // Invocation de la mthode tearDown() ; 
            try {
               tearDown.invoke(unObjet, new Object[0]) ; 
            } catch (Throwable throwable) {
               ErrorMessage.invocation(tearDown, throwable) ;
            }

         }
      }
   }

   // Gnration de numberOfTests mthodes de tests, qui effectuent
   // chacune numberOfMethodCalls appels de mthode.
   public void generate
         (int numberOfTests, int numberOfMethodCalls) {
      try {
         generate2(numberOfTests, numberOfMethodCalls) ; 
      } catch (IOException io) {
         ErrorMessage.ioWriteError(fileName) ; 
      }
   }

   private void generate2
      (int numberOfTests, int numberOfMethodCalls)
   throws IOException {

      Constructor consToExtend = null ; 
      Object unObjet = null ; 
      
      if (classTester.getToExtendClass() != null) {

         consToExtend = classTester.getConsToExtend() ; 

         try {
            unObjet = consToExtend.newInstance(new Class[0]) ; 
         } catch (Throwable throwable) {
            ErrorMessage.invocation(consToExtend, throwable) ; 
         } 
      }

      writeHeader() ; 
      writeClass() ; 
      writeMain(numberOfTests, numberOfMethodCalls) ; 

      for (int i = 1 ; i <= numberOfTests ; i++) {
        
         executeSetUp(unObjet) ; 

         Chaine.ecrire(file, 
            "   /** \n" + 
            "    * Test method number " + i + " \n" + 
            "    */ \n\n" + 
            "   public void test" + i + "() throws Exception { \n") ; 

         Chaine.ecrire(file,
            "      int _nb_calls = 0 ; \n") ; 

         if (classTester.getForJUnit()) {
         } else if (setUp != null) {
            Chaine.ecrire(file, "      setUp() ; \n") ; 
         }

         Chaine.ecrire(file, "      try { \n") ; 
         
         generateNCalls(numberOfMethodCalls) ; 

         executeTearDown(unObjet) ; 


         if (classTester.getForJUnit()) {
            Chaine.ecrire(file, 
               "      } catch (Throwable _except) {\n" + 
               "         fail(\"\\n        \" + _except.getMessage()) ; \n" + 
               "      } \n") ; 
         } else {
            Chaine.ecrire(file,
               "      } catch (Throwable _except) {\n" + 
               "         error(_except, " + i + ", _nb_calls) ; \n" + 
               "      } \n") ; 

            if (tearDown != null) {
               Chaine.ecrire(file, "      tearDown() ; \n") ; 
            }
         }

         Chaine.ecrire(file, "   } // test" + i + " \n\n") ; 
         classTester.leDictionnaire.viderObjets() ; 

      }

      Chaine.ecrire(file, "} // " + className + "\n") ; 
      file.close() ; 
      System.out.println("File " + fileName + " generated.") ; 
      classTester.leDictionnaire.compteRendu() ; 
      classTester.leDictionnaire.reinit() ; 

   }
}

